﻿using Moq;
using Models.Game;
using Models.Interfaces;
using System.Reflection;
using static System.Type;

namespace Tests;

public class GameTests
{
    private readonly Mock<IPersistence> _mockPersistence; // Mocking (faking) the persistence layer to allow for testing without a persistent connection
    private readonly Game _game;

    public GameTests()
    {
        _mockPersistence = new Mock<IPersistence>();
        _game = new Game(_mockPersistence.Object);
    }

    [Fact]
    public void AddPlayer_ShouldAddPlayerToList()
    {
        var player = new Player();

        _game.AddPlayer(player);

        Assert.Contains(player, _game.Players);
    }

    [Fact]
    public void AddGame_ShouldAddGameToList()
    {
        var game = new Game(new Player(), new Map("test_background"));

        _game.AddGame(game);

        Assert.Contains(game, _game.Games);
    }

    [Fact]
    public void AddMap_ShouldAddMapToList()
    {
        var map = new Map("test_background.png");

        _game.AddMap(map);

        Assert.Contains(map, _game.Maps);
    }

    [Fact]
    public void AddBestScore_ShouldAddBestScoreToList()
    {
        var bestScore = new BestScore(3, 125);

        _game.AddBestScore(bestScore);

        Assert.Contains(bestScore, _game.BestScores);
    }

    [Fact]
    public void LoadData_ShouldLoadDataFromPersistence()
    {
        var players = new List<Player> { new Player() };
        var games = new List<Game> { new Game(new Player(), new Map("test_background")) };
        var maps = new List<Map> { new Map("test_background") };
        var bestScores = new List<BestScore> { new BestScore(1, 26) };

        _mockPersistence.Setup(p => p.LoadData()).Returns((players, games, maps, bestScores));

        _game.LoadData();

        Assert.Equal(players, _game.Players);
        Assert.Equal(games, _game.Games);
        Assert.Equal(maps, _game.Maps);
        Assert.Equal(bestScores, _game.BestScores);
    }

    
    /*
    [Fact]
    public void Game_Initialization_SetsMap()
    {
        Player player = new Player("test_player");
        Map map = new Map("test_background");

        var game = new Game(player, map);

        Assert.Equal(map, game.UsedMap);
    }

    [Fact]
    public void Game_Initialization_SetsDice()
    {
        Player player = new Player("test_player");
        Map map = new Map("test_background");

        var game = new Game(player, map);

        Assert.NotNull(game.Dice1);
        Assert.NotNull(game.Dice2);
    }

    [Fact]
    public void Game_Initialization_SetsGameRules()
    {
        Player player = new Player("test_player");
        Map map = new Map("test_background");

        var game = new Game(player, map);

        Assert.NotNull(game.GameRules);
    }

    [Fact]
    public void RollAllDice_RollsAllDice()
    {
        Player player = new Player("test_player");
        Map map = new Map("test_background");

        var game = new Game(player, map);

        game.RollAllDice();

        Assert.True(game.Dice1.Value >= 0 && game.Dice1.Value <= 5);
        Assert.True(game.Dice2.Value >= 1 && game.Dice2.Value <= 6);
    }

    [Fact]
    public void MarkOperationAsChecked_Check_Well()
    {
        Player player = new Player("test_player");
        Map map = new Map("test_background");

        var game = new Game(player, map);

        game.MarkOperationAsChecked(Operation.LOWER);

        Assert.True(game.UsedMap.OperationGrid[0].IsChecked);
    }

    */
}
